h1 {
    font-size: 2em;
    padding: 0.5em 0;    
}

h2 {
    font-size: 1.3em;
    padding: 0.5em 0;
}

.searchInput {
    padding: 0.5em 0;
    text-align: center;
}

.load {
    display: none;
}

ul {
    padding: 0.5em 0;
    margin-left: 2em;
}

li {
    list-style-type: circle;
}

.generatedBy {
    margin: 10em 0;
    padding: 0.3em 0;
    font-size: 75%;
    text-align: center;
    border-top: 1px solid #aaaaaa;
}
